/*
 * AudioBusManager.h 
 *
 * Copyright (c) 2008 Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License version 2.1 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Lesser GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License version 2.1
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
/**
 * \defgroup audiobusman Audio Bus Manager
 */
/**
 * \ingroup audiobusman
 * \file
 *    Audio Bus Manager APIs.
 *
 * \version $Revision: 1.8 $
 * \date    $Date: 2008/07/12 00:59:37 $
 */

#ifndef __AUDIO_BUS_MANAGER_H__
#define __AUDIO_BUS_MANAGER_H__

/**************************************************************************/
/*                               Defines                                  */
/**************************************************************************/
#define ABM_PLUGIN_STORE_FOLDER "/usr/lib/audiomanagement/abm/plugins"

/**************************************************************************/
/*                                                                        */
/*  Function Name: ABM_Initialize                                         */
/*                                                                        */
/**************************************************************************/
/**
 *  Function to initializes the Audio Bus Manager module. This function 
 *  allocates the data structures to load audio bus plugins and
 *  loads the plugin shared libraries if they are available in the 
 *  ABM_PLUGIN_STORE_FOLDER   
 *                                                                        
 *  \return                                                              
 *     TRUE, if succeeded in intialization and loading atleast one plugin.
 *     FALSE, if faiulre occured during intialization.                         
 *                                                                        
 **************************************************************************
 */

gboolean ABM_Initialize ();

/**************************************************************************/
/*                                                                        */
/*  Function Name: ABM_Terminate                                          */
/*                                                                        */
/**************************************************************************/
/**
 *  Function to terminate the Audio Bus Manager module. This function 
 *  de-allocates the data structures and unloads the currently loaded audio
 *  bus plugins.                                          
 *  \return                                                              
 *     TRUE, if succeeded terminating ABM and uloading it plugins.
 *     FALSE, if faiulre occured during the terminate call.                         
 *                                                                        
 **************************************************************************
 */
gboolean ABM_Terminate ();

/**************************************************************************/
/*                                                                        */
/*  Function Name: ABM_Allocate                                           */
/*                                                                        */
/**************************************************************************/
/**
 *  Function to allocate audio channel and return a pulse audio device index.                                    
 *  \return                                                              
 *     ABM_SUCCESS, if success.
 *     ABM_FAILURE, if failure.                       
 *                                                                        
 **************************************************************************
 */
ABM_Results_t ABM_Allocate
(
    ABM_AudioPluginParams_t AudioPluginParams, /**< [In] Audio Parameters  
                                               used toallocate the audio 
                                               channels on the audio bus*/
    GHashTable *AudioParamsTable, /**< [In] Audio Parameters need to 
                                      allocate the audio channels on the
                                      audio bus*/
    ABM_AudioSetupCb_t pAllocateCb /**< [In] Allocate callback 
                                        to return the result of allocation*/
);

/**************************************************************************/
/*                                                                        */
/*  Function Name: ABM_Allocate                                           */
/*                                                                        */
/**************************************************************************/
/**
 *  Function to allocate audio channel and return a pulse audio device index.                                    
 *  \return                                                              
 *     ABM_SUCCESS, if success.
 *     ABM_FAILURE, if failure.                       
 *                                                                        
 **************************************************************************
 */
ABM_Results_t ABM_DeAllocate
(
    ABM_AudioPluginParams_t AudioPluginParams, /**< [In] Audio Parameters  
                                                  used to allocate the audio 
                                                  channels on the audio bus*/
    GHashTable *AudioParamsTable, /**< [In] Audio Parameters need to 
                                       disconnect the audio channels on the
                                       audio bus*/

    ABM_AudioSetupCb_t pDeAllocateCb /**< [In] DeAllocate callback 
                                             to return the result of 
                                             de-allocation*/
);

/**************************************************************************/
/*                                                                        */
/*  Function Name: ABM_Connect                                           */
/*                                                                        */
/**************************************************************************/
/**
 *  Function to connect an audio sync to an audio channel that has 
 *  already been allocated.                                    
 *  \return                                                              
 *     ABM_SUCCESS, if success.
 *     ABM_FAILURE, if failure.                       
 *                                                                        
 **************************************************************************
 */
ABM_Results_t ABM_Connect
(
    ABM_AudioPluginParams_t AudioPluginParams, /**< [In] Audio Parameters  
                                               used to connect the audio 
                                               channels on the audio bus*/
    GHashTable *AudioParamsTable, /**< [In] Audio Parameters need to 
                                      connect the audio channels on the
                                      audio bus*/
    ABM_AudioSetupCb_t pConnectCb /**< [In] Connect callback 
                                        to return the result of connect
                                        operation*/
);

/**************************************************************************/
/*                                                                        */
/*  Function Name: ABM_Disconnect                                         */
/**************************************************************************/
/**
 * Function to disconnect an audio sync to an audio channel that has 
 *  already been allocated and connected.                                     
 *  \return                                                              
 *     ABM_SUCCESS, if succeeded.
 *     ABM_FAILURE, if faiulre occured.                       
 *                                                                        
 **************************************************************************
 */
ABM_Results_t ABM_Disconnect
(
    ABM_AudioPluginParams_t AudioPluginParams, /**< [In] Audio Parameters  
                                                        used to connect the audio 
                                                        channels on the audio bus*/
    GHashTable *AudioParamsTable, /**< [In] Audio Parameters need to 
                                       disconnect the audio channels on the
                                       audio bus*/

    ABM_AudioSetupCb_t pDisconnectCb /**< [In] Allocate callback 
                                              to return the result of 
                                              disconnect operation*/
);
#endif /* __PLUGIN_MANAGER_H__ */
