/*
 * PluginData.h 
 *
 * Copyright (c) 2008 Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License version 2.1 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Lesser GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License version 2.1
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
/**
 * \ingroup audiobusman
 * \file
 *    Data strcture used in storing plugin data for Audio Management related
 *     plugins
 *
 *  
 *
 * \version $Revision: 1.6 $
 * \date    $Date: 2008/07/12 00:59:37 $
 */
#ifndef __PLUGIN_DATA_H__
#define __PLUGIN_DATA_H__

/**************************************************************************/
/*                              Includes                                  */
/**************************************************************************/
#include <glib.h>
#include <gmodule.h>

/**************************************************************************/
/*                               Defines                                  */
/**************************************************************************/
/** Plugin type name length */

#define PLUGIN_TYPE_NAME_LENGTH 25

/**************************************************************************/
/*                              Typedefs                                  */
/**************************************************************************/
// Holds data associated with an audio management plugin
typedef struct
{
    gboolean Loaded;
    const gchar *pFileName;                    /**< name of the plugin (actual file of plugin) */
    gchar PluginType[PLUGIN_TYPE_NAME_LENGTH]; /**< enumeration identifying the plugin type */
    guint Version;                             /**< version og the plugin */
    GModule *pModule;                          /**< pointer to a gmodule object that holds the plugin */
    gpointer pInitFn;                          /**< Intialization function for teh plugin */
    gpointer pPluginFnTable;                   /**< Plugin Function Table */
} PluginData_t;
#endif                                         //__PLUGIN_DATA_H__
