/*
 * PluginManager.c 
 *
 * Copyright (c) 2008 Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License version 2.1 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Lesser GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License version 2.1
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
/**
/// @cond internal_documentation
 * \file
 *    Plugin Manager Interface
 *
 *  This is just some text
 *
 * \version $Revision: 1.6 $
 * \date    $Date: 2008/07/12 00:59:37 $
 */

/**************************************************************************/
/*                              Includes                                  */
/**************************************************************************/
#include <gmodule.h>
#include <glib/gprintf.h>

#include "PluginManager.h"

/**************************************************************************/
/*                               Defines                                  */
/**************************************************************************/

/**************************************************************************/
/*                              Typedefs                                  */
/**************************************************************************/

/**************************************************************************/
/*                              Globals                                   */
/**************************************************************************/

/**************************************************************************/
/*                              Locals                                   */
/**************************************************************************/

/**************************************************************************/
/*                       Private Functions Bodies                         */
/**************************************************************************/

/**************************************************************************/
/*                        Public Functions Bodies                         */
/**************************************************************************/

gboolean PM_LoadPlugin
(
    PluginData_t *pPlugin, const char *pPluginInitFnName
)
{
    gboolean Success = TRUE;
    gpointer pFnInit;

    if (NULL == pPlugin->pFileName || 0 == pPlugin->pFileName[0])
    {
        g_printf ("PluginManager: Name is empty\n");
        return FALSE;
    }

    pPlugin->pModule = g_module_open (pPlugin->pFileName, 0);

    if (NULL == pPlugin->pModule)
    {
        g_printf ("PluginManager: Error opening %s. Reported error: %s\n", pPlugin->pFileName, g_module_error ());
        return FALSE;
    }

    if (!g_module_symbol (pPlugin->pModule, pPluginInitFnName, &pFnInit))
    {
        g_printf ("PluginManager: Error %s: %s\n", pPlugin->pFileName, g_module_error ());
        g_module_close (pPlugin->pModule);
        pPlugin->pModule = NULL;
        return FALSE;
    }

    pPlugin->pInitFn = pFnInit;
    pPlugin->Loaded = TRUE;
    g_printf ("PluginManager:  Loaded plugin: %s\n", pPlugin->pFileName);
    return Success;
} 

void PM_UnloadPlugin
(
    PluginData_t *pPlugin
)
{
    g_module_close (pPlugin->pModule);
    pPlugin->pModule = NULL;
}
/// @endcond
