/*
 * AudioBusManagerPlugin.h 
 *
 * Copyright (c) 2008 Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License version 2.1 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Lesser GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License version 2.1
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
/**
 * \ingroup audiobusman
 * \file
 *    Audio Bus Manager plugin API
 *
 *  This header provides Audio Manager interfaces for plugins
 *
 * \version $Revision: 1.16 $
 * \date    $Date: 2008/07/12 00:59:38 $
 */

#ifndef __AUDIO_BUS_MANAGER_PLUGIN_H__
#define __AUDIO_BUS_MANAGER_PLUGIN_H__

/**************************************************************************/
/*                              Includes                                  */
/**************************************************************************/
#include <memory.h>
#include <string.h>
#include <glib.h>
#include <dbus/dbus-glib.h>
#include "../PluginData.h"

/**************************************************************************/
/*                               Defines                                  */
/**************************************************************************/
#define ABM_PLUGIN_INIT_FN_NAME "ABM_PluginInit"

/** Maximum plugins supported by ABM */
#define ABM_MAX_PLUGINS 3

/** channel related defines */
#define ABM_MAX_CHANNELS 32

/** max length of the plugin type name */
#define ABM_MAX_PLUGIN_TYPE_NAME_LENGTH 25

/** max length of the client type name */
#define ABM_MAX_CLIENT_TYPE_NAME_LENGTH 25

/** device id string length  */
#define ABM_MAX_DEVICE_ID_LENGTH 200

/** Casting to get ABM plugin specific type from pPluginFnTable of PluginData_t structure */
#define PLUGIN_DATA_FUNC_TABLE ((ABM_PluginFunctions_t *)pPluginData->pPluginFnTable)

#define ABM_INVALID_PA_INDEX  -1

#define ABM_AUDIO_IS_SINK(x)  \
  ((strcmp(x,"sink") == 0) || (strcmp(x, "local_sink") == 0 ) ? TRUE : FALSE)
#define ABM_AUDIO_IS_LOCAL(x) \
    ((strcmp(x,"local_source") == 0) || (strcmp(x, "local_sink") == 0 ) ? TRUE : FALSE)

typedef enum
{
    ABM_SUCCESS = 0,
    ABM_FAILURE = -1,
}


ABM_Results_t;


typedef int ABM_PulseAudioDevIndex;
typedef int ABM_AlsaDevIndex;


typedef struct
{
    gchar DeviceID[ABM_MAX_DEVICE_ID_LENGTH];
    gchar PluginType[ABM_MAX_PLUGIN_TYPE_NAME_LENGTH];
    gchar ClientType[ABM_MAX_PLUGIN_TYPE_NAME_LENGTH];
    DBusGMethodInvocation *pDbusContext;
} ABM_AudioPluginParams_t;

/* Version to return from init function.  This should be incremented
* each time the plug-in API changes between binary releases.
*/
#define ABM_PLUGIN_VERSION 1

typedef void (*ABMU_PA_ModuleLoadHandler)(int PluginParamIndex,
int PA_ModuleIndex,
int PA_DeviceIndex,
int IsSink,
int AlsaDevice,
int AlsaSubDevice);


typedef void (*ABM_AudioSetupCb_t)(ABM_AudioPluginParams_t AudioPluginParams,
gboolean success,
GHashTable *returnValue,
ABM_PulseAudioDevIndex PA_Index);


typedef gboolean (*ABM_PluginInit_t) (PluginData_t *pPluginData);

/** Function table returned by the plug-in on initialisation.
* Most of the parameters are passed through from the Device Manager
* API so see the API fn headers for details of valid parameter values.
*/
typedef struct
{
    gboolean (* FnTerminate) (void);

    void (* FnAllocateChannels) (ABM_AudioPluginParams_t AudioPluginParams, GHashTable * pAudioParamsTable, ABM_AudioSetupCb_t pResultCb);

    void (* FnDeAllocateChannels) (ABM_AudioPluginParams_t AudioPluginParams, GHashTable * AudioParamsTable, ABM_AudioSetupCb_t pResultCb);

    void (* FnConnectChannels) (ABM_AudioPluginParams_t AudioPluginParams, GHashTable * pAudioParamsTable, ABM_AudioSetupCb_t pResultCb);

    void (* FnDisconnectChannels) (ABM_AudioPluginParams_t AudioPluginParams, GHashTable * AudioParamsTable, ABM_AudioSetupCb_t pResultCb);
} ABM_PluginFunctions_t;
#endif /* __AUDIO_BUS_MANAGER_PLUGIN_H__ */
