/*
 * acmd_errors.c 
 *
 * Copyright (c) 2008 Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License version 2.1 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Lesser GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License version 2.1
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
/// @cond internal_documentation
/**
 * \file 
 *    Initialization for dbus error codes returned from the acmd.    
 *
 * \version $Revision: 1.8 $
 * \date    $Date: 2008/07/12 01:27:17 $
 */

/**************************************************************************/
/*                              Includes                                  */
/**************************************************************************/
#include "acmd.h"

/**************************************************************************/
/*                               Defines                                  */
/**************************************************************************/

/**************************************************************************/
/*                              Typedefs                                  */
/**************************************************************************/

/**************************************************************************/
/*                              Externs                                   */
/**************************************************************************/

/**************************************************************************/
/*                              Globals                                   */
/**************************************************************************/

/**************************************************************************/
/*                           Local Variables                              */
/**************************************************************************/
//the list of errors

static GError *errorPtrList[ACM_LAST_ERROR];


static gboolean errorsInitialized = FALSE;

/**************************************************************************/
/*  Private Functions Prototypes-functions used internally in this file   */
/**************************************************************************/

/**************************************************************************/
/*                       Public Functions Bodies                         */
/**************************************************************************/

gboolean error_init (
)
{
    gboolean success = TRUE;
    gchar *domain_str;

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "OK");
    errorPtrList[ACM_ERROR_OK] = g_error_new (g_quark_from_string (domain_str), ACM_ERROR_OK, "OK");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "Failed");
    errorPtrList[ACM_ERROR_FAILED] = g_error_new (g_quark_from_string (domain_str), ACM_ERROR_FAILED, "Failed");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "NotConnected");
    errorPtrList[ACM_ERROR_NOT_CONNECTED] = g_error_new (g_quark_from_string (domain_str), ACM_ERROR_NOT_CONNECTED,
                                                         "NotConnected");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "AlreadyConnected");
    errorPtrList[ACM_ERROR_ALREADY_CONNECTED] = g_error_new (
                                                    g_quark_from_string (domain_str), ACM_ERROR_ALREADY_CONNECTED,
                                                    "AlreadyConnected");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "UnableToAllocate");
    errorPtrList[ACM_ERROR_UNABLE_TO_ALLOCATE] = g_error_new (
                                                     g_quark_from_string (domain_str), ACM_ERROR_UNABLE_TO_ALLOCATE,
                                                     "Unable to allocate");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "UnableToDeAllocate");
    errorPtrList[ACM_ERROR_UNABLE_TO_DEALLOCATE] = g_error_new (
                                                       g_quark_from_string (domain_str), ACM_ERROR_UNABLE_TO_DEALLOCATE,
                                                       "Unable to deallocate");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "NotAllocated");
    errorPtrList[ACM_ERROR_NOT_ALLOCATED] = g_error_new (g_quark_from_string (domain_str), ACM_ERROR_NOT_ALLOCATED,
                                                         "Not allocated");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "OperationInProgress");
    errorPtrList[ACM_ERROR_OPERATION_IN_PROGRESS] = g_error_new (
                                                        g_quark_from_string (domain_str),
                                                        ACM_ERROR_OPERATION_IN_PROGRESS,
                                                        "Operation in progress");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "UnknownDevice");
    errorPtrList[ACM_ERROR_UNKNOWN_DEVICE] = g_error_new (g_quark_from_string (domain_str), ACM_ERROR_UNKNOWN_DEVICE,
                                                          "Unknown device");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "UnknownBus");
    errorPtrList[ACM_ERROR_UNKNOWN_BUS] = g_error_new (g_quark_from_string (domain_str), ACM_ERROR_UNKNOWN_BUS,
                                                       "Unknown bus");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "NonLocalDevice");
    errorPtrList[ACM_ERROR_NONLOCAL_DEVICE] = g_error_new (g_quark_from_string (domain_str), ACM_ERROR_NONLOCAL_DEVICE,
                                                           "Non Local Device");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "UnknownClientType");
    errorPtrList[ACM_ERROR_NONLOCAL_DEVICE] = g_error_new (
                                                  g_quark_from_string (domain_str), ACM_ERROR_UNKNOWN_CLIENT_TYPE,
                                                  "Unknown Client Type");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "InvalidDeviceId");
    errorPtrList[ACM_ERROR_NONLOCAL_DEVICE] = g_error_new (g_quark_from_string (domain_str), ACM_ERROR_INVALID_DEV_ID,
                                                           "InvalidDeviceId");
    g_free (domain_str);

    domain_str = g_strdup_printf ("%s.%s", ACM_ERROR_PATH, "UnableToSendControlMessageToDevice");
    errorPtrList[ACM_ERROR_UNABLE_TO_SEND_CONTROL_MESSAGE_TO_DEVICE] = g_error_new (
                                                                           g_quark_from_string (domain_str),
                                                                           ACM_ERROR_UNABLE_TO_SEND_CONTROL_MESSAGE_TO_DEVICE,
                                                                           "UnableToSendControlMessageToDevice");
    g_free (domain_str);

    errorsInitialized = TRUE;

    return success;
}


gboolean error_finalize (
)
{
    gboolean success = TRUE;
    gint i = 0;

    if (errorsInitialized)
    {
        for (i = 0; i < (ACM_LAST_ERROR); i++)
        {
            g_error_free (errorPtrList[i]);
        }
        errorsInitialized = FALSE;
    }
    else
        success = FALSE;

    return success;
} GError *getGError
(
    enum ACM_Errors result /**< [in] The result to translate to a GError.*/
)
{
    GError *retError = NULL;

    if (errorsInitialized)
    {
        /* Make sure that the passed in value is in the range of known errors.
         * Otherwise return the generic "failed" error.
         */
        if ((result >= 0) && (result <= ACM_LAST_ERROR))
        {
            retError = errorPtrList[result];
        }
        else
        {
            retError = errorPtrList[ACM_ERROR_FAILED];
        }
    }

    return retError;
}

/**************************************************************************/
/*                       Private Functions Bodies                         */
/**************************************************************************/

/// @endcond
