/*
 * acmd_errors.h 
 *
 * Copyright (c) 2008 Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License version 2.1 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Lesser GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License version 2.1
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
/**
 * \ingroup audioconnman_dbus_interfaces
 * \file 
 * This file contains the error enumerations and accessor functions for converting
 * those enumerated values to GErrors for use with DBus.  
 *
 * \version $Revision: 1.9 $
 * \date    $Date: 2008/07/12 01:27:17 $
 */

#ifndef ACMD_ERRORS_H_
#define ACMD_ERRORS_H_

/**************************************************************************/
/*                              Includes                                  */
/**************************************************************************/

/**************************************************************************/
/*                               Defines                                  */
/**************************************************************************/

/**************************************************************************/
/*                              Typedefs                                  */
/**************************************************************************/
/** List of errors for the Audio Connection Manager. */

enum ACM_Errors
{
    ACM_ERROR_OK,                                       /**< Indicates no error.*/
    ACM_ERROR_FAILED,                                   /**< Generic failure.*/
    ACM_ERROR_NOT_CONNECTED,                            /**< The device is not connected.*/
    ACM_ERROR_ALREADY_CONNECTED,                        /**< The device is already connected.*/
    ACM_ERROR_UNABLE_TO_ALLOCATE,                       /**< Unable to allocate a channel on a bus.*/
    ACM_ERROR_UNABLE_TO_DEALLOCATE,                     /**< Unable to deallocate a channel on a bus.*/
    ACM_ERROR_NOT_ALLOCATED,                            /**< The specified channel is not allocated.*/
    ACM_ERROR_OPERATION_IN_PROGRESS,                    /**< Bus operation in progress.*/
    ACM_ERROR_UNKNOWN_DEVICE,                           /**< The specified device is not registered with ACM.*/
    ACM_ERROR_UNKNOWN_BUS,                              /**< The specified bus is not supported by ABM.*/
    ACM_ERROR_UNKNOWN_CLIENT_TYPE,                      /**< The specified client type is not supported by ABM.*/
    ACM_ERROR_INVALID_DEV_ID,                           /**< The specified device id is invalid.*/
    ACM_ERROR_NONLOCAL_DEVICE,                          /**< Connect or disconnect was called on a non-local device */
    ACM_ERROR_UNABLE_TO_SEND_CONTROL_MESSAGE_TO_DEVICE, /**< Could not send a control message to the specified device */
    ACM_LAST_ERROR
} ;

/**************************************************************************/
/*                      Functions Declarations                            */
/**************************************************************************/
/**************************************************************************/
/*                                                                        */
/*  Function Name: error_init                                             */
/*                                                                        */
/**************************************************************************/
/**
 *  Sets up error list.  All GErrors used by the DBus objects should be
 *  created in this function and accessed using getGError().               
 *
 *  \return TRUE if successful.                 
 *  \see getGError
 *                                                     
 **************************************************************************
 */
gboolean error_init
(
    void
);

/**************************************************************************/
/*                                                                        */
/*  Function Name: error_finalize                                         */
/*                                                                        */
/**************************************************************************/
/**
 *  Frees all of the GErrors in the error list.               
 *
 *  \return TRUE if successful.                 
 *                                                     
 **************************************************************************
 */
gboolean error_finalize
(
    void
);

/**************************************************************************/
/*                                                                        */
/*  Function Name: getGError                                              */
/*                                                                        */
/**************************************************************************/
/**
 *  Accessor function for the array of GErrors to be returned by DBus methods.
 *  If the input parameter is outside of the range of the enumeration, the "Failed"
 *  GError is returned instead.               
 *
 *  \return A pointer to the GError associated with the given ACM_Errors value. 
 *          The return value may be NULL if the error list has not been initialized.                
 *                                                     
 **************************************************************************
 */
GError *getGError
(
    enum ACM_Errors result /**< [in] The result to translate to a GError.*/
);
#endif                     /*ACMD_ERRORS_H_*/
