/*
 * devicecontrol.c
 *
 * Copyright (c) 2008 Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License version 2.1 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Lesser GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License version 2.1
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/// @cond internal_documentation
/**
 * \file 
 * This file contains the implementation of the message handler functions
 * for the com.windriver.DC DBus interface.
 *
 * \version $Revision: 1.3 $
 * \date    $Date: 2008/07/12 01:47:25 $
 */

/**************************************************************************/
/*                              Includes                                  */
/**************************************************************************/
#include "devicecontrol.h"
#include "device_control_module-glue.h"

/**************************************************************************/
/*                               Defines                                  */
/**************************************************************************/

G_DEFINE_TYPE
(
    DeviceControl, devicecontrol, G_TYPE_OBJECT
);

/**************************************************************************/
/*                              Externs                                   */
/**************************************************************************/
extern DBusGConnection *dbus_connection;
extern gboolean dc_initialized;

/**************************************************************************/
/*                              Globals                                   */
/**************************************************************************/

/**************************************************************************/
/*                          Local Variables                               */
/**************************************************************************/

/**************************************************************************/
/*  Private Functions Prototypes-functions used internally in this file   */
/**************************************************************************/

/**************************************************************************/
/*                       Private Functions Bodies                         */
/**************************************************************************/
/***********************************************************************
 * Object Control functions for the managed classes.
 *
 * These function must be in the same file as where the GType is defined.
 ***********************************************************************/
/**************************************************************************/
/*                                                                        */
/*  Function Name: devicecontrol_class_init                                         */
/*                                                                        */
/**************************************************************************/
/**
 *  This function is called the first time that a new DeviceControl object
 * is created.  It sets up any required variables for the DeviceControlClass.
 * In this case this function just sets up the signals that can be emited by
 * instances of this class.
 *                                                     
 **************************************************************************
 */
void devicecontrol_class_init
(
    DeviceControlClass *klass /**< [in] The class object to init.*/
)
{
    // Define all signals for the interface.
    klass->DCSignals[DC_HELLO_CALLED] = g_signal_new (
                                            "hello_called",                        G_OBJECT_CLASS_TYPE (klass),
                                            (G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION), 0,
                                            NULL,                                  NULL,
                                            g_cclosure_marshal_VOID__STRING,       G_TYPE_NONE,
                                            1,                                     G_TYPE_STRING);
}
/**************************************************************************/
/*                                                                        */
/*  Function Name: devicecontrol_init                                               */
/*                                                                        */
/**************************************************************************/
/**
 *  This function is called on each new DeviceControl object when it is created.
 * This function is effectively the constructor for this class, all variables needed
 * should be set up here.                             
 *                                                     
 **************************************************************************
 */
void devicecontrol_init
(
    DeviceControl *server /**< [in] The object instance to init.*/
)
{
    if (!dc_initialized)
    {
        dbus_g_object_type_install_info (devicecontrol_get_type (), &dbus_glib_device_control_module_object_info);

        dbus_g_connection_register_g_object (dbus_connection, DC_OBJECT_PATH, G_OBJECT (server));
        dc_initialized = TRUE;
    }
    else
    {
        g_critical ("DeviceControl already created!");
    }
}
/**************************************************************************/
/*                                                                        */
/*  Function Name: devicecontrol_finalize                                           */
/*                                                                        */
/**************************************************************************/
/**
 *  This function is called when a DeviceControl object is freed.  This function
 * is effectively the destructor for this class, any local variables stored in 
 * this object's struct should be freed here.                           
 *                                                     
 **************************************************************************
 */
void devicecontrol_finalize
(
    DeviceControl *server /**< [in] The object instance to finalize.*/
)
{
/* nothing for now, but this function must exist because G_DEFINE_TYPE
 * expects it.
 */
}
/***********************************************************************
 * DBus message handler functions
 ***********************************************************************/

gboolean device_control_module_mute
(
    DeviceControl *server,         /**< [in] the object instance that the method was called on. */
    DBusGMethodInvocation *context /**< [in] the method invocation context. */
)
{
    g_message ("Mute called\n");
    dbus_g_method_return (context);
    return TRUE;
}


gboolean device_control_module_un_mute
(
    DeviceControl *server,         /**< [in] the object instance that the method was called on. */
    DBusGMethodInvocation *context /**< [in] the method invocation context. */
)
{
    g_message ("Un_mute called\n");
    dbus_g_method_return (context);
    return TRUE;
}


gboolean device_control_module_start
(
    DeviceControl *server,         /**< [in] the object instance that the method was called on. */
    DBusGMethodInvocation *context /**< [in] the method invocation context. */
)
{
    g_message ("Start called\n");
    dbus_g_method_return (context);
    return TRUE;
}


gboolean device_control_module_stop
(
    DeviceControl *server,         /**< [in] the object instance that the method was called on. */
    DBusGMethodInvocation *context /**< [in] the method invocation context. */
)
{
    g_message ("Stop called\n");
    dbus_g_method_return (context);
    return TRUE;
} 

gboolean device_control_module_pause
(
    DeviceControl *server,         /**< [in] the object instance that the method was called on. */
    DBusGMethodInvocation *context /**< [in] the method invocation context. */
)
{
    g_message ("Pause called\n");
    dbus_g_method_return (context);
    return TRUE;
}

/// @endcond
