/*
 * psm.h
 *
 * Copyright (c) 2008 Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License version 2.1 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Lesser GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License version 2.1
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


/// @cond internal_documentation
/**
 * \file 
 * This file contains the declarations of the message handler functions
 * for the com.windriver.PSM DBus interface    
 *
 * \version $Revision: 1.2 $
 * \date    $Date: 2008/07/12 01:27:19 $
 */
#ifndef _DEVICE_CONTROL_H_
#define _DEVICE_CONTROL_H_

/**************************************************************************/
/*                              Includes                                  */
/**************************************************************************/
#include <glib.h>
#include <glib/gmessages.h>
#include <dbus/dbus-glib-bindings.h>

/**************************************************************************/
/*                               Defines                                  */
/**************************************************************************/
/** The name of the DBus interface for the power state manager. */
#define DC_INTERFACE "com.windriver.audio.DeviceControlModule"

/**************************************************************************/
/*                               Globals                                  */
/**************************************************************************/
/** The base DBus service name for the power state manager.*/

extern char DC_DBUS_SERVICE[];

/** The DBus object path for the power state manager. */
extern char DC_OBJECT_PATH[];

/**************************************************************************/
/*                              Typedefs                                  */
/**************************************************************************/

/** List of signals for the DEVICE_CONTROL interface. */
enum
{
    DC_HELLO_CALLED, /**< HelloCalled*/
    DC_LAST_SIGNAL
} ;

/** Structure representing the DC DBus object. */
typedef struct
{
    GObject parent; /**< The object's parent object. */
} DeviceControl;

/** Structure representing the class for the DC DBus object. */
typedef struct
{
    GObjectClass parent; /**< The class's parent object. */
    guint DCSignals[DC_LAST_SIGNAL]; /**< An array of the registered 
                                            signal IDs for this class.*/
} DeviceControlClass;

/**************************************************************************/
/*                          Functions Declarations                        */
/**************************************************************************/
/**************************************************************************/
/*                                                                        */
/*  Function Name: psm_get_type                                       */
/*                                                                        */
/**************************************************************************/
/**
 *  This function is provided by the G_DEFINE_TYPE macro.  It is used when 
 *  creating a new object and registering it on the system bus.               
 *
 *  \return the type of the GObject.                 
 *                                                     
 **************************************************************************
 */
GType devicecontrol_get_type (
);

/***********************************************************************
 * DBus message handler functions
 ***********************************************************************/

gboolean device_control_module_mute
(
    DeviceControl *server,         /**< [in] the object instance that the method was called on. */
    DBusGMethodInvocation *context /**< [in] the method invocation context. */
);


gboolean device_control_module_un_mute
(
    DeviceControl *server,         /**< [in] the object instance that the method was called on. */
    DBusGMethodInvocation *context /**< [in] the method invocation context. */
);


gboolean device_control_module_start
(
    DeviceControl *server,         /**< [in] the object instance that the method was called on. */
    DBusGMethodInvocation *context /**< [in] the method invocation context. */
);


gboolean device_control_module_stop
(
    DeviceControl *server,         /**< [in] the object instance that the method was called on. */
    DBusGMethodInvocation *context /**< [in] the method invocation context. */
);gboolean device_control_module_pause
(
    DeviceControl *server,         /**< [in] the object instance that the method was called on. */
    DBusGMethodInvocation *context /**< [in] the method invocation context. */
);

/// @endcond
#endif //_SERIAL_MANAGER_H_
