<?xml version="1.0"?>

<xsl:stylesheet  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>

  <xsl:variable name="indent" select="'    '"/>
  <xsl:variable name="newline" select="'&#10;'"/>

  <xsl:template match="/node">
    <xsl:apply-templates select="interface"/>
  </xsl:template>

  <xsl:template match="interface">
    <xsl:value-of select="preceding-sibling::comment()[1]"/>
    <xsl:text>class </xsl:text>
    <!--<xsl:value-of select="translate(@name,'.','_')"/> -->
    <xsl:value-of select="@name"/>
    <xsl:text>{</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$indent"/>
    <xsl:text>public:</xsl:text>
    <xsl:apply-templates select="method|signal"/>
    <xsl:text>};</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="method">
    <xsl:value-of select="preceding-sibling::comment()[1]"/>
    <xsl:choose>
      <xsl:when test="arg[@direction = 'out'][1]/@type">
        <xsl:apply-templates select="arg[@direction = 'out'][1]/@type"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>void</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text> </xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:text>(</xsl:text>
    <xsl:apply-templates select="arg[@direction = 'in']|arg[@direction = 'out'][position() > 1]"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$indent"/>
    <xsl:text>);</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="signal">
    <xsl:value-of select="preceding-sibling::comment()[1]"/>
    <xsl:text>void</xsl:text>
    <xsl:text> </xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:text>(</xsl:text>
    <xsl:apply-templates select="arg[@direction = 'in']|arg[@direction = 'out']"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$indent"/>
    <xsl:text>);</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>


  <xsl:template match="arg">
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$indent"/>
    <xsl:value-of select="$indent"/>
    <xsl:apply-templates select="@type"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>
    <xsl:text> </xsl:text>
    <xsl:value-of select="preceding-sibling::comment()[1]"/>
  </xsl:template>

  <xsl:template match="arg[@direction = 'out']">
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$indent"/>
    <xsl:value-of select="$indent"/>
    <xsl:apply-templates select="@type"/>
    <xsl:text> &amp;</xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>
    <xsl:text> </xsl:text>
    <xsl:value-of select="preceding-sibling::comment()[1]"/>
  </xsl:template>

  

  <xsl:template match="@type[. = 'y']">
    <xsl:text>byte</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'ay']">
    <xsl:text>byte[]</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'b']">
    <xsl:text>boolean</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'ab']">
    <xsl:text>boolean[]</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'n']">
    <xsl:text>int16</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'an']">
    <xsl:text>int16[]</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'q']">
    <xsl:text>uint16</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'aq']">
    <xsl:text>uint16[]</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'i']">
    <xsl:text>int32</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'ai']">
    <xsl:text>int32[]</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'u']">
    <xsl:text>uint32</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'au']">
    <xsl:text>uint32[]</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'x']">
    <xsl:text>int64</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'ax']">
    <xsl:text>int64[]</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 't']">
    <xsl:text>uint64</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'at']">
    <xsl:text>uint64[]</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'd']">
    <xsl:text>double</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'ad']">
    <xsl:text>double[]</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 's']">
    <xsl:text>string</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'as']">
    <xsl:text>string[]</xsl:text>
  </xsl:template>
  
  <xsl:template match="@type[. = 'a{ss}']">
    <xsl:text>dict</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'u']">
    <xsl:text>uint32</xsl:text>
  </xsl:template>

  <xsl:template match="@type[. = 'au']">
    <xsl:text>uint32[]</xsl:text>
  </xsl:template>
  
  <xsl:template match="@type[. = 'v']">
    <xsl:text>variant</xsl:text>
  </xsl:template>
</xsl:stylesheet>
