/*
 * paclient.c 
 *
 * Copyright (c) 2008 Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License version 2.1 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Lesser GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License version 2.1
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
/*
 * \file 
 *    PulseAudio Routing Client.    
 *
 * \version $Revision: 1.15 $
 * \date    $Date: 2008/07/12 00:56:52 $
 */

/**************************************************************************/
/*                              Includes                                  */
/**************************************************************************/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "paclient.h"

/**************************************************************************/
/*                               Defines                                  */
/**************************************************************************/
#define CONTEXT_OPERATION_CHECK_DEAD(ctx,operation)                           \
    if(!ctx->pPACtx || pa_context_get_state(ctx->pPACtx) != PA_CONTEXT_READY) \
    {                                                                         \
        fprintf(stderr,"pulseaudio context died\n");                          \
        if(operation)                                                         \
        {                                                                     \
            pa_operation_unref(operation);                                    \
        }                                                                     \
        pa_threaded_mainloop_unlock(ctx->pMainLoop);                          \
        return PA_CLIENT_TERMINATED;                                          \
    }
#define CONTEXT_OPERATION_LOOP(ctx,operation)                                 \
    if(!o)                                                                    \
    {                                                                         \
        pa_threaded_mainloop_unlock(ctx->pMainLoop);                          \
    fprintf(stderr,"operation failed\n");                                     \
    return PA_CLIENT_ERR;                                                     \
    }                                                                         \
    while(pa_operation_get_state(operation) != PA_OPERATION_DONE)             \
    {                                                                         \
        pa_threaded_mainloop_wait(ctx->pMainLoop);                            \
        CONTEXT_OPERATION_CHECK_DEAD(pCtx,operation);                         \
    }                                                                         \
    pa_operation_unref(o);


#if 1
#define PACLIENT_DEBUG_LOG(fmt,arg...)
#else
#define PACLIENT_DEBUG_LOG printf
#endif
#define TRUE 1
#define FALSE 0
#define PA_DONT_BLOCK 0

#undef ENABLE_PACLIENT_GET_STREAMINDEX_BY_PROPERTY
#undef ENABLE_PACLIENT_MOVE_STREAM

/**************************************************************************/
/*                              Typedefs                                  */
/**************************************************************************/

typedef struct SuccessCallbackData
{
    pa_threaded_mainloop *pMainLoop;
    int success; /* Success from callback */
} SuccessCallbackData_t;


typedef struct IndexCallbackData
{
    pa_threaded_mainloop *pMainLoop;
    uint32_t index; /* Index from callback */
} IndexCallbackData_t;


typedef struct GetDevIndexFromModuleCBData
{
    pa_threaded_mainloop *pMainLoop;
    uint32_t moduleIndex; /* Input to callback */
    uint32_t deviceIndex; /* return device index */
    PA_Client_Err_t status;
} GetDevIndexFromModuleData_t;


typedef struct GetStreamIndexFromPropertyData
{
    pa_threaded_mainloop *pMainLoop;
    const char *propKey;   /* property key */
    const char *propValue; /* property value to search for */
    uint32_t streamIndex; /* return  index */
    PA_Client_Err_t status;
} GetStreamIndexFromPropertyData_t;


typedef struct MoveAllStreamsByPropertyCallbackData
{
    pa_threaded_mainloop *pMainLoop;
    const char *propKey;     /* property key */
    const char *propValue;   /* property value to search for */
    uint32_t deviceIndex;   /* index of device to move stream to */
    PA_Client_Err_t status; /* return status */
} MoveAllStreamsByPropertyCallbackData_t;

/**************************************************************************/
/*                              Externs                                   */
/**************************************************************************/

/**************************************************************************/
/*                              Globals                                   */
/**************************************************************************/

/**************************************************************************/
/*                           Local Variables                              */
/**************************************************************************/

/**************************************************************************/
/*  Private Functions Prototypes-functions used internally in this file   */
/**************************************************************************/
static void PAContextStateCallback
(
    pa_context *c, void *userdata
);


static void PAContextSuccessCallback
(
    pa_context *c, int success, void *userdata
);


static void PAIndexCallback
(
    pa_context *c, uint32_t idx, void *userdata
);


static void PASinkInfoFromModuleCallback
(
    pa_context *c, const pa_sink_info *i, int eol, void *userdata
);


static void PASrcInfoFromModuleCallback
(
    pa_context *c, const pa_source_info *i, int eol, void *userdata
);


#ifdef ENABLE_PACLIENT_GET_STREAMINDEX_BY_PROPERTY
static void PASinkInputInfoFromPropertyCallback
(
    pa_context *c, const pa_sink_input_info *i, int eol, void *userdata
);


static void PASrcOutputInfoFromPropertyCallback
(
    pa_context *c, const pa_source_output_info *i, int eol, void *userdata
);
#endif

static void PAClientMoveSourceOutputsByPropertyCallback
(
    pa_context *c, const pa_source_output_info *i, int eol, void *userdata
);


static void PAClientMoveSinkInputsByPropertyCallback
(
    pa_context *c, const pa_sink_input_info *i, int eol, void *userdata
);

/**************************************************************************/
/*                       Public Functions Bodies                         */
/**************************************************************************/
PA_Client_Err_t PAClientCtxInit
(
    PA_Client_Ctx_t *pCtx, /**< [in] Pointer to unintialized Client Context struct */
    const char *clientName  /**< [in] Pointer to string to be copied to PulseAudio  */
)
{
    pa_mainloop_api *mainLoopAPI;

    if (!clientName)
    {
        fprintf (stderr, "PAClientCtxInit failed, no clientName\n");
        return PA_CLIENT_ERR;
    }
    /* Set up a new main loop */
    if (!(pCtx->pMainLoop = pa_threaded_mainloop_new ()))
    {
        fprintf (stderr, "pa_mainloop_new() failed.\n");
        return PA_CLIENT_ERR;
    }

    mainLoopAPI = pa_threaded_mainloop_get_api (pCtx->pMainLoop);

    /* Create a new connection context */
    if (!(pCtx->pPACtx = pa_context_new (mainLoopAPI, clientName)))
    {
        fprintf (stderr, "pa_context_new() failed.\n");
        pa_threaded_mainloop_free (pCtx->pMainLoop);
        return PA_CLIENT_ERR;
    }

    pa_context_set_state_callback (pCtx->pPACtx, PAContextStateCallback, pCtx->pMainLoop);

    if (pa_threaded_mainloop_start (pCtx->pMainLoop))
    {
        fprintf (stderr, "pa_mainloop_run() failed.\n");
        pa_threaded_mainloop_free (pCtx->pMainLoop);
        pa_context_unref (pCtx->pPACtx);
        pCtx->pPACtx = NULL;
        return PA_CLIENT_ERR;
    }

    return PA_CLIENT_OK;
}


PA_Client_Err_t PAClientCtxConnect
(
    PA_Client_Ctx_t *pCtx /**< [in] Pointer to intialized Client Context struct */
)
{
    int exitLoop;
    PA_Client_Err_t ret = PA_CLIENT_ERR;
    pa_context_state_t ctxState;

    ctxState = pa_context_get_state (pCtx->pPACtx);

    if ((ctxState == PA_CONTEXT_READY))
    {
        return PA_CLIENT_OK;
    }

    if (ctxState != PA_CONTEXT_UNCONNECTED)
    {
        PAClientCtxCleanup (pCtx);
        PAClientCtxInit (pCtx, "PAClient Reconnect\n");
    }
    /* Connect the context */
    pa_threaded_mainloop_lock (pCtx->pMainLoop);

    if (-1 == pa_context_connect (pCtx->pPACtx, NULL, PA_CONTEXT_NOAUTOSPAWN, NULL))
    {
        pa_threaded_mainloop_unlock (pCtx->pMainLoop);
        fprintf (stderr, "pa_context_connect fails\n");
        return PA_CLIENT_ERR;
    }

    exitLoop = FALSE;

    while (!exitLoop)
    {
        pa_threaded_mainloop_wait (pCtx->pMainLoop);
        ctxState = pa_context_get_state (pCtx->pPACtx);
        switch (ctxState)
        {
            case PA_CONTEXT_CONNECTING:
            case PA_CONTEXT_AUTHORIZING:
            case PA_CONTEXT_SETTING_NAME:
                break;
            case PA_CONTEXT_TERMINATED:
                ret = PA_CLIENT_TERMINATED;
                exitLoop = TRUE;
                break;
            case PA_CONTEXT_FAILED:
                ret = PA_CLIENT_CONNECT_FAILED;
                exitLoop = TRUE;
                break;
            case PA_CONTEXT_READY:
                exitLoop = 1;
                ret = PA_CLIENT_OK;
                break;

            default:
                fprintf (stderr, "unknown bad context state %d\n", ctxState);
                ret = PA_CLIENT_ERR;
                exitLoop = TRUE;
                break;
                break;
        }
    }

    pa_threaded_mainloop_unlock (pCtx->pMainLoop);
    return ret;
}


PA_Client_Err_t PAClientCheckConnection
(
    PA_Client_Ctx_t *pCtx /**< [in] Pointer to Client Context struct */
)
{
    pa_context_state_t ctxState;
    ctxState = pa_context_get_state (pCtx->pPACtx);

    if (PA_CONTEXT_IS_GOOD (ctxState))
    {
        return PA_CLIENT_OK;
    }

    return PA_CLIENT_ERR;
}


PA_Client_Err_t PAClientCtxLoadModule
(
    PA_Client_Ctx_t *pCtx, /**< [in] Pointer to Client Context struct */
    const char *moduleName, /**< [in] Name of PulseAudio module to load */
    const char *argument,   /**< [in] Module arguments */
    uint32_t *moduleIndex /**< [out] Pointer to update with loaded module index 
                           on success */
)
{
    IndexCallbackData_t *pCallbackData, callbackData;
    pa_operation *o;

    pa_threaded_mainloop_lock (pCtx->pMainLoop);

    pCallbackData = &callbackData;

    pCallbackData->pMainLoop = pCtx->pMainLoop;
    o = pa_context_load_module (pCtx->pPACtx, moduleName, argument, PAIndexCallback, pCallbackData);

    CONTEXT_OPERATION_LOOP (pCtx, o);

    *moduleIndex = pCallbackData->index;

    pa_threaded_mainloop_accept (pCtx->pMainLoop);

    pa_threaded_mainloop_unlock (pCtx->pMainLoop);

    return PA_CLIENT_OK;
}


PA_Client_Err_t PAClientCtxUnloadModule
(
    PA_Client_Ctx_t *pCtx, /**< [in] Pointer to Client Context struct */
    uint32_t moduleIndex   /**< [in] Index of Module to unload */
)
{
    SuccessCallbackData_t *pSuccessCbData, successCbData;
    pa_operation *o;
    PA_Client_Err_t ret;

    pa_threaded_mainloop_lock (pCtx->pMainLoop);

    pSuccessCbData = &successCbData;

    pSuccessCbData->pMainLoop = pCtx->pMainLoop;
    o = pa_context_unload_module (pCtx->pPACtx, moduleIndex, PAContextSuccessCallback, pSuccessCbData);

    CONTEXT_OPERATION_LOOP (pCtx, o);

    if (pSuccessCbData->success)
    {
        ret = PA_CLIENT_OK;
    }
    else
    {
        ret = PA_CLIENT_ERR;
    }

    pa_threaded_mainloop_accept (pCtx->pMainLoop);
    pa_threaded_mainloop_unlock (pCtx->pMainLoop);

    return ret;
}


PA_Client_Err_t PAClientGetDevFromModule
(
    PA_Client_Ctx_t *pCtx,           /**< [in] Pointer to Client Context struct */
    const uint32_t moduleIndex,      /**< [in] Index of Module */
    PA_CLIENT_DEVICE_TYPE_t devType, /**< [in] Type of device that is connected to module */
    uint32_t *deviceIndex            /**< [out] Location that result is copied to on success */
)
{
    GetDevIndexFromModuleData_t *pCbData, cbData;
    pa_operation *o;
    PA_Client_Err_t ret;

    pCbData = &cbData;

    pa_threaded_mainloop_lock (pCtx->pMainLoop);
    pCbData->pMainLoop = pCtx->pMainLoop;
    pCbData->moduleIndex = moduleIndex;
    pCbData->status = PA_CLIENT_NO_MATCHING_DEVICE;

    if (devType == PA_CLIENT_DEVICE_SINK)
    {
        o = pa_context_get_sink_info_list (pCtx->pPACtx, PASinkInfoFromModuleCallback, pCbData);
    }
    else
    {
        o = pa_context_get_source_info_list (pCtx->pPACtx, PASrcInfoFromModuleCallback, pCbData);
    }

    CONTEXT_OPERATION_LOOP (pCtx, o);

    *deviceIndex = pCbData->deviceIndex;

    pa_threaded_mainloop_accept (pCtx->pMainLoop);
    pa_threaded_mainloop_unlock (pCtx->pMainLoop);

    ret = pCbData->status;

    return ret;
}


PA_Client_Err_t PAClientCtxCleanup
(
    PA_Client_Ctx_t *pCtx /**< [in] Pointer to Client Context struct */
)
{
    if (pCtx->pPACtx)
    {
        pa_context_disconnect (pCtx->pPACtx);
        pa_context_unref (pCtx->pPACtx);
        pCtx->pPACtx = NULL;
    }

    if (pCtx->pMainLoop)
    {
        pa_threaded_mainloop_stop (pCtx->pMainLoop);
        pa_threaded_mainloop_free (pCtx->pMainLoop);
        pCtx->pMainLoop = NULL;
    }

    return PA_CLIENT_OK;
}


#ifdef ENABLE_PACLIENT_MOVE_STREAM
PA_Client_Err_t PAClientMoveStream
(
    PA_Client_Ctx_t *pCtx,              /**< [in] Pointer to Client Context struct */
    PA_CLIENT_STREAM_TYPE_t streamType, /**< [in] Type of stream to move   */
    const uint32_t streamIndex,         /**< [in] Index of stream to move */
    const uint32_t devIndex             /**< [in] Index of device to move to */
)
{
    PA_Client_Err_t ret;
    SuccessCallbackData_t *pSuccessCbData, successCbData;
    pa_operation *o;

    pSuccessCbData = &successCbData;
    pSuccessCbData->pMainLoop = pCtx->pMainLoop;
    pa_threaded_mainloop_lock (pCtx->pMainLoop);

    if (streamType == PA_CLIENT_SINK_INPUT)
    {
        o = pa_context_move_sink_input_by_index (pCtx->pPACtx, streamIndex, devIndex, PAContextSuccessCallback,
                                                 pSuccessCbData);
    }
    else
    {
        o = pa_context_move_source_output_by_index (pCtx->pPACtx, streamIndex, devIndex, PAContextSuccessCallback,
                                                    pSuccessCbData);
    }

    CONTEXT_OPERATION_LOOP (pCtx, o);

    if (pSuccessCbData->success)
    {
        ret = PA_CLIENT_OK;
    }
    else
    {
        ret = PA_CLIENT_ERR;
    }

    pa_threaded_mainloop_accept (pCtx->pMainLoop);
    pa_threaded_mainloop_unlock (pCtx->pMainLoop);

    return ret;
}
#endif

PA_Client_Err_t PAClientEventSubscribe
(
    PA_Client_Ctx_t *pCtx,              /**< [in] Pointer to Client Context struct */
    pa_context_subscribe_cb_t callback, /**< [in] Event callback to be passed to PulseAudio */
    void *cbData,                       /**< [in] Userdata to be passed to the callback */
    pa_subscription_mask_t eventMask    /**< [in] PulseAudio Event Mask                */
)
{
    PA_Client_Err_t ret;
    SuccessCallbackData_t *pSuccessCbData, successCbData;
    pa_operation *o;

    pSuccessCbData = &successCbData;

    pSuccessCbData->pMainLoop = pCtx->pMainLoop;

    pa_threaded_mainloop_lock (pCtx->pMainLoop);

    pa_context_set_subscribe_callback (pCtx->pPACtx, callback, cbData);

    o = pa_context_subscribe (pCtx->pPACtx, eventMask, PAContextSuccessCallback, pSuccessCbData);

    CONTEXT_OPERATION_LOOP (pCtx, o);

    if (pSuccessCbData->success)
    {
        ret = PA_CLIENT_OK;
    }
    else
    {
        ret = PA_CLIENT_ERR;
    }

    pa_threaded_mainloop_accept (pCtx->pMainLoop);
    pa_threaded_mainloop_unlock (pCtx->pMainLoop);

    return ret;
}


#ifdef ENABLE_PACLIENT_GET_STREAMINDEX_BY_PROPERTY
PA_Client_Err_t PAClientGetStreamIndexByProperty
(
    PA_Client_Ctx_t *pCtx,              /**< [in] Pointer to Client Context struct */
    const char *propKey,                 /**< [in] Pulse Property to look up */
    const char *propValue,               /**< [in] Property Value to look for */
    PA_CLIENT_STREAM_TYPE_t streamType, /**< [in] Stream type to search for */
    uint32_t *streamIndex               /**< [out] Updated with stream index if found */
)
{
    GetStreamIndexFromPropertyData_t *pCbData, cbData;
    pa_operation *o;
    PA_Client_Err_t ret;

    pa_threaded_mainloop_lock (pCtx->pMainLoop);
    pCbData = &cbData;

    pCbData->pMainLoop = pCtx->pMainLoop;
    pCbData->propKey = propKey;
    pCbData->propValue = propValue;
    pCbData->status = PA_CLIENT_NO_MATCHING_STREAM;

    if (streamType == PA_CLIENT_SINK_INPUT)
    {
        o = pa_context_get_sink_input_info_list (pCtx->pPACtx, PASinkInputInfoFromPropertyCallback, pCbData);
    }
    else
    {
        o = pa_context_get_source_output_info_list (pCtx->pPACtx, PASrcOutputInfoFromPropertyCallback, pCbData);
    }

    CONTEXT_OPERATION_LOOP (pCtx, o);

    *streamIndex = pCbData->streamIndex;

    pa_threaded_mainloop_accept (pCtx->pMainLoop);
    pa_threaded_mainloop_unlock (pCtx->pMainLoop);

    ret = pCbData->status;

    return ret; /* returns PA_CLIENT_OK if matched, or PA_CLIENT_NO_MATCHING_STREAM */
}
#endif

PA_Client_Err_t PAClientMoveAllStreamsByProperty
(
    PA_Client_Ctx_t *pCtx,              /**< [in] Pointer to Client Context struct */
    PA_CLIENT_STREAM_TYPE_t streamType, /**< [in] Type of streams to move   */
    const char *pPropertyKey,            /**< [in] PulseProperty to use as the stream id */
    const char *pStreamId,               /**< [in] Id of streams to move */
    const uint32_t devIndex             /**< [in] Index of device to move to */
)
{
    MoveAllStreamsByPropertyCallbackData_t *pCallbackData, callbackData;
    pa_operation *o;
    PA_Client_Err_t ret = PA_CLIENT_OK;

    pa_threaded_mainloop_lock (pCtx->pMainLoop);
    pCallbackData = &callbackData;

    pCallbackData->pMainLoop = pCtx->pMainLoop;
    pCallbackData->propKey = pPropertyKey;
    pCallbackData->propValue = pStreamId;
    pCallbackData->deviceIndex = devIndex;
    pCallbackData->status = PA_CLIENT_OK;

    if (streamType == PA_CLIENT_SINK_INPUT)
    {
        o = pa_context_get_sink_input_info_list (pCtx->pPACtx, PAClientMoveSinkInputsByPropertyCallback, pCallbackData);
    }
    else
    {
        o = pa_context_get_source_output_info_list (pCtx->pPACtx, PAClientMoveSourceOutputsByPropertyCallback,
                                                    pCallbackData);
    }

    CONTEXT_OPERATION_LOOP (pCtx, o);

    ret = pCallbackData->status;

    pa_threaded_mainloop_accept (pCtx->pMainLoop);
    pa_threaded_mainloop_unlock (pCtx->pMainLoop);

    return ret;
}
/**************************************************************************/
/*                       Private Functions Bodies                         */
/*************************************************************************/

static void PAContextSuccessCallback
(
    pa_context *c, int success, void *userdata
)
{
    SuccessCallbackData_t *pSuccessCbData;
    pa_threaded_mainloop *m;

    pSuccessCbData = (SuccessCallbackData_t *)userdata;

    m = pSuccessCbData->pMainLoop;
    pSuccessCbData->success = success;

    if (m)
    {
        pa_threaded_mainloop_signal (m, PA_DONT_BLOCK);
    }
}


#ifdef ENABLE_PACLIENT_GET_STREAMINDEX_BY_PROPERTY
static void PASinkInputInfoFromPropertyCallback
(
    pa_context *c, const pa_sink_input_info *i, int eol, void *userdata
)
{
    GetStreamIndexFromPropertyData_t *pCbData;
    const void *prop;
    size_t length;

    pCbData = (GetStreamIndexFromPropertyData_t *)userdata;

    PACLIENT_DEBUG_LOG ("PASinkInputFromProp callback eol %d\n", eol);

    if (!eol)
    {
        PACLIENT_DEBUG_LOG ("searching for %s\n", pCbData->propKey);
        if (!pa_proplist_get (i->proplist, pCbData->propKey, &prop, &length))
        {
            PACLIENT_DEBUG_LOG ("PASinkInputInfo from property %s\n", (char *)prop);
            if (strncmp (pCbData->propValue, prop, length) == 0)
            {
                pCbData->status = PA_CLIENT_OK;
                pCbData->streamIndex = i->index;
            }
        }
    }
    else
    {
        pa_threaded_mainloop_signal (pCbData->pMainLoop, PA_DONT_BLOCK);
    }
}


static void PASrcOutputInfoFromPropertyCallback
(
    pa_context *c, const pa_source_output_info *i, int eol, void *userdata
)
{
    GetStreamIndexFromPropertyData_t *pCbData;
    const void *prop;
    size_t length;

    pCbData = (GetStreamIndexFromPropertyData_t *)userdata;

    if (!eol)
    {
        if (!pa_proplist_get (i->proplist, pCbData->propKey, &prop, &length))
        {
            if (strncmp (pCbData->propValue, prop, length) == 0)
            {
                pCbData->status = PA_CLIENT_OK;
                pCbData->streamIndex = i->index;
            }
        }
    }
    else
    {
        pa_threaded_mainloop_signal (pCbData->pMainLoop, PA_DONT_BLOCK);
    }
}
#endif

static void PASinkInfoFromModuleCallback
(
    pa_context *c, const pa_sink_info *i, int eol, void *userdata
)
{
    GetDevIndexFromModuleData_t *pCbData;

    pCbData = (GetDevIndexFromModuleData_t *)userdata;

    if (!eol)
    {
        if (i->owner_module == pCbData->moduleIndex)
        {
            pCbData->status = PA_CLIENT_OK;
            pCbData->deviceIndex = i->index;
        }
    }
    else
    {
        pa_threaded_mainloop_signal (pCbData->pMainLoop, PA_DONT_BLOCK);
    }
}


static void PASrcInfoFromModuleCallback
(
    pa_context *c, const pa_source_info *i, int eol, void *userdata
)
{
    GetDevIndexFromModuleData_t *pCbData;

    pCbData = (GetDevIndexFromModuleData_t *)userdata;

    if (!eol)
    {
        if (i->owner_module == pCbData->moduleIndex)
        {
            pCbData->status = PA_CLIENT_OK;
            pCbData->deviceIndex = i->index;
        }
    }
    else
    {
        pa_threaded_mainloop_signal (pCbData->pMainLoop, PA_DONT_BLOCK);
    }
}


static void PAIndexCallback
(
    pa_context *c, uint32_t idx, void *userdata
)
{
    pa_threaded_mainloop *m;
    IndexCallbackData_t *pIndexData;

    pIndexData = (IndexCallbackData_t *)userdata;

    m = pIndexData->pMainLoop;

    pIndexData->index = idx;

    if (m)
    {
        pa_threaded_mainloop_signal (m, PA_DONT_BLOCK);
    }
}


static void PAContextStateCallback
(
    pa_context *c, void *userdata
)
{
    pa_threaded_mainloop *m;

    m = (pa_threaded_mainloop *)userdata;

    if (m)
    {
        pa_threaded_mainloop_signal (m, PA_DONT_BLOCK);
    }


    #if 0
    PACLIENT_DEBUG_LOG ("context state callback ctx state %d\n", pa_context_get_state (c));
    #endif

}


static void PAClientMoveSinkInputsByPropertyCallback
(
    pa_context *c, const pa_sink_input_info *i, int eol, void *userdata
)
{
    MoveAllStreamsByPropertyCallbackData_t *pCbData;
    const void *prop;
    size_t length;

    pCbData = (MoveAllStreamsByPropertyCallbackData_t *)userdata;

    PACLIENT_DEBUG_LOG ("PAClientMoveSinkInputsByProperty callback eol %d\n", eol);

    if (!eol)
    {
        PACLIENT_DEBUG_LOG ("searching for %s\n", pCbData->propKey);
        if (!pa_proplist_get (i->proplist, pCbData->propKey, &prop, &length))
        {
            if (strncmp (pCbData->propValue, prop, length) == 0)
            {
                if (i->sink != pCbData->deviceIndex)
                {
                    if (!pa_context_move_sink_input_by_index (c, i->index, pCbData->deviceIndex, NULL, NULL))
                    {
                        pCbData->status = PA_CLIENT_ERR;
                    }
                }
            }
        }
    }
    else
    {
        pa_threaded_mainloop_signal (pCbData->pMainLoop, PA_DONT_BLOCK);
    }
} static void PAClientMoveSourceOutputsByPropertyCallback
(
    pa_context *c, const pa_source_output_info *i, int eol, void *userdata
)
{
    MoveAllStreamsByPropertyCallbackData_t *pCbData;
    const void *prop;
    size_t length;

    pCbData = (MoveAllStreamsByPropertyCallbackData_t *)userdata;

    PACLIENT_DEBUG_LOG ("PAClientMoveSourceOutputsByProperty callback eol %d\n", eol);

    if (!eol)
    {
        PACLIENT_DEBUG_LOG ("searching for %s\n", pCbData->propKey);
        if (!pa_proplist_get (i->proplist, pCbData->propKey, &prop, &length))
        {
            if (strncmp (pCbData->propValue, prop, length) == 0)
            {
                if (i->source != pCbData->deviceIndex)
                {
                    if (!pa_context_move_source_output_by_index (c, i->index, pCbData->deviceIndex, NULL, NULL))
                    {
                        pCbData->status = PA_CLIENT_ERR;
                    }
                }
            }
        }
    }
    else
    {
        pa_threaded_mainloop_signal (pCbData->pMainLoop, PA_DONT_BLOCK);
    }
}
