/*
 * parouting_test.c 
 *
 * Copyright (c) 2008 Wind River Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License version 2.1 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Lesser GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License version 2.1
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
#include <stdio.h>
#include <string.h>
#include "parouting.h"

int main
(
    int argv, char *argc[]
)
{
    char c;
    PA_Routing_Ctx_t *pRCtx = NULL;
    PA_Routing_Err_t err;
    PA_Client_Err_t client_err;
    printf ("PARouting Test, Enter Test Number:\n");

    while ((c = getchar ()))
    {
        switch (c)
        {
            case '1':
                pRCtx = PARoutingModuleNew ("testid");
                printf ("Test %c: PARoutingModuleNew: ", c);
                if (!pRCtx)
                    printf ("FAILED\n");
                else
                    printf ("SUCCESS\n");
                break;
            case '2':
                err = PARoutingPAConnect (pRCtx);
                printf ("Test %c: PARoutingPAConnect: ", c);
                if (err == PA_ROUTING_OK)
                    if (PARoutingModuleCheckPAConnection (pRCtx) == PA_ROUTING_OK)
                        printf ("SUCCESS\n");
                    else
                        printf ("FAILED\n");
                else
                    printf ("FAILED\n");
                break;
            case '3':
                err = PARoutingPADisconnect (pRCtx);
                printf ("Test %c: PARoutingPADisconnect: ", c);
                if (err == PA_ROUTING_OK)
                    printf ("SUCCESS\n");
                else
                    printf ("FAILED\n");
                break;
            case '4':
                err = PARoutingModuleFree (pRCtx);
                printf ("Test %c: PARoutingModuleFree: ", c);
                if (err == PA_ROUTING_OK)
                    printf ("SUCCESS\n");
                else
                    printf ("FAILED\n");
                break;
            case '5':
            {
                uint32_t expectedDeviceIndex;
                uint32_t actualDeviceIndex;
                char *streamName = "\\test\\object\\name";
                char *streamName2 = "inputStreamName";
                expectedDeviceIndex = 22;
                printf ("Test %c, Set/Get Output Routes: ", c);
                err = PARoutingModuleSetRoute (pRCtx, PA_ROUTE_OUTPUT, streamName, expectedDeviceIndex);

                if (err != PA_ROUTING_OK)
                {
                    printf ("FAILED\n");
                    break;
                }

                err = PARoutingModuleGetRoute (pRCtx, PA_ROUTE_OUTPUT, streamName, strlen (streamName),
                                               &actualDeviceIndex);

                if ((err != PA_ROUTING_OK) || (expectedDeviceIndex != actualDeviceIndex))
                {
                    printf ("FAILED\n");
                    break;
                }
                else
                    printf ("SUCCESS\n");

                printf ("Test %c, Set/Get Input Routes: ", c);
                expectedDeviceIndex = 33;
                err = PARoutingModuleSetRoute (pRCtx, PA_ROUTE_INPUT, streamName2, expectedDeviceIndex);

                if (err != PA_ROUTING_OK)
                {
                    printf ("FAILED\n");
                    break;
                }

                err = PARoutingModuleGetRoute (pRCtx, PA_ROUTE_INPUT, streamName2, strlen (streamName2),
                                               &actualDeviceIndex);

                if ((err != PA_ROUTING_OK) || (expectedDeviceIndex != actualDeviceIndex))
                {
                    printf ("FAILED\n");
                    break;
                }
                else
                    printf ("SUCCESS\n");

                printf ("Test %c, Get Invalid Route: ", c);
                err = PARoutingModuleGetRoute (pRCtx, PA_ROUTE_OUTPUT, streamName2, strlen (streamName2),
                                               &actualDeviceIndex);

                if (err != PA_ROUTING_OK)
                    printf ("SUCCESS\n");
                else
                    printf ("FAILED returned PA_ROUTING_OK for invalid Get\n");

                expectedDeviceIndex = 44;
                err = PARoutingModuleSetRoute (pRCtx, PA_ROUTE_OUTPUT, "test44", expectedDeviceIndex);

                if (err != PA_ROUTING_OK)
                {
                    printf ("FAILED\n");
                    break;
                }

                expectedDeviceIndex = 55;
                err = PARoutingModuleSetRoute (pRCtx, PA_ROUTE_OUTPUT, "test55", expectedDeviceIndex);

                if (err != PA_ROUTING_OK)
                {
                    printf ("FAILED\n");
                    break;
                }

                if (PA_ROUTING_OK == PARoutingModuleDumpRoutes (pRCtx))
                    printf ("SUCCESS\n");
                else
                {
                    printf ("FAILED\n");
                    break;
                }

                printf ("Test %c, Apply Routes: ", c);

                if (PA_ROUTING_OK == PARoutingModuleApplyRoutes (pRCtx))
                    printf ("SUCCESS\n");
                else
                {
                    printf ("FAILED\n");
                    break;
                }

                printf ("Test %c, Remove Routes: ", c);
                err = PARoutingModuleRemoveRoute (pRCtx, PA_ROUTE_INPUT, streamName2);

                if (err == PA_ROUTING_OK)
                    printf ("SUCCESS\n");
                else
                    printf ("FAILED\n");

                err = PARoutingModuleRemoveRoute (pRCtx, PA_ROUTE_OUTPUT, streamName);

                if (err == PA_ROUTING_OK)
                    printf ("SUCCESS\n");
                else
                    printf ("FAILED\n");
            }
                break;
            case '6':
            {
                uint32_t moduleIndex;
                uint32_t srcIndex;
                uint32_t sinkIndex;
                printf ("Test %c: PAClientCtxLoadModule: ", c);

                if (PA_CLIENT_OK == PAClientCtxLoadModule (pRCtx->pClientCtx, "module-null-sink", NULL, &moduleIndex))
                    if (PA_CLIENT_OK == PAClientGetDevFromModule (pRCtx->pClientCtx, moduleIndex, PA_CLIENT_DEVICE_SINK,
                                                                  &sinkIndex))
                        if (PA_CLIENT_OK
                                == PAClientGetDevFromModule (pRCtx->pClientCtx, moduleIndex, PA_CLIENT_DEVICE_SOURCE,
                                                             &srcIndex))
                            printf ("SUCCESS Loaded Module %d, sink %d, source %d\n", moduleIndex, sinkIndex, srcIndex);
                        else
                            printf ("FAILED\n");
                    else
                        printf ("FAILED\n");
                else
                    printf ("FAILED\n");

                printf ("Test %c: PAClientCtxUnloadModule: ", c);

                if (PA_CLIENT_OK == PAClientCtxUnloadModule (pRCtx->pClientCtx, moduleIndex))
                    printf ("SUCCESS\n");
                else
                    printf ("FAILED\n");

                break;
            }
            case '7':
                printf ("Test %c: PARoutingModuleCheckPAConnection: ", c);
                if (PARoutingModuleCheckPAConnection (pRCtx) == PA_ROUTING_OK)
                    printf ("SUCCESS\n");
                else
                    printf ("FAILED\n");
                break;
            case '8':
                printf ("Test %c: PARoutingModuleSetAndApplyRoute: ", c);
            {
                uint32_t expectedDeviceIndex;
                char *streamName = "\\test\\object\\name";
                printf ("Test %c, Set/Get Output Routes: ", c);
                expectedDeviceIndex = 2;
                err = PARoutingModuleSetAndApplyRoute (pRCtx, PA_ROUTE_OUTPUT, streamName, expectedDeviceIndex);

                if (err != PA_ROUTING_OK)
                {
                    printf ("FAILED\n");
                    break;
                }

                printf ("SUCCESS\n");
                break;
            }
            case '9':
                printf ("Test %c: PARoutingModuleApplyAllRoutes: ", c);
            {
                err = PARoutingModuleApplyRoutes (pRCtx);

                if (err != PA_ROUTING_OK)
                {
                    printf ("FAILED\n");
                    break;
                }

                printf ("SUCCESS\n");
                break;
            }
            case 'a':
                printf ("Test %c: PAClientMoveAllStreamsByProperty: ", c);
            {
                client_err = PAClientMoveAllStreamsByProperty (pRCtx->pClientCtx, PA_CLIENT_SINK_INPUT, "acmdeviceid",
                                                               "mp3",             1);

                if (client_err != PA_CLIENT_OK)
                {
                    printf ("FAILED\n");
                    break;
                }

                printf ("SUCCESS\n");
                break;
            }
            case 'b':
                printf ("Test %c: PAClientMoveAllStreamsByProperty: ", c);
            {
                client_err = PAClientMoveAllStreamsByProperty (pRCtx->pClientCtx, PA_CLIENT_SOURCE_OUTPUT,
                                                               "acmdeviceid",     "mp3",
                                                               2);

                if (client_err != PA_CLIENT_OK)
                {
                    printf ("FAILED\n");
                    break;
                }

                printf ("SUCCESS\n");
                break;
            }

            case 'X':
            case 'x':
                printf ("Exiting\n");
                return 0;
                break;
        }
    }

    return 0;
}
